from enum import Enum


class Region(Enum):
    EU = "https://icobrain-eu.icometrix.com"
    US = "https://icobrain-us.icometrix.com"
    CA = "https://icobrain-ca.icometrix.com"
    AU = "https://icobrain-au.icometrix.com"
    STUDIES = "https://studies.icometrix.com"
