class IcometrixException(Exception):
    ...


class PolicyException(IcometrixException):
    ...


class HashAlgorithmException(IcometrixException):
    ...


class HashSizeException(IcometrixException):
    ...
